@@ -145,12 +145,37 @@ test_that("SQLServer", {
     dbWriteTable(con, tblName, values, field.types = list(c1 = "INT", c2 = "VARCHAR(MAX)", c3 = "INT", c4 = "TEXT"))
     on.exit(dbRemoveTable(con, tblName))
     received <- DBI::dbReadTable(con, tblName)
+    # Also test retrival using a prepared statement
+    received2 <- dbGetQuery(con,
+      paste0("SELECT * FROM ", tblName, "  WHERE c1 = ?"), params = list(1L))
     expect_equal(values, received)
+    expect_equal(values, received2)
+  })
+
+  local({
+    con <- DBItest:::connect(DBItest:::get_default_context())
+    tblName <- "test_na"
+    # With SELECT ing with the OEM SQL Server driver, everything
+    # after the first column should be unbound. Test null detection for
+    # unbound columns (NULL is registered after a call to nanodbc::result::get)
+    values <- data.frame(
+      c1 = c("this is varchar max", NA_character_),
+      c2 = c(1L, NA_integer_),
+      c3 = c(1.0, NA_real_),
+      c4 = c(TRUE, NA),
+      c5 = c(Sys.Date(), NA),
+      c6 = c(Sys.time(), NA),
+      stringsAsFactors = FALSE)
+    dbWriteTable(con, tblName, values, field.types = list(c1 = "VARCHAR(MAX)", c2 = "INT", c3 = "FLOAT", c4 = "BIT", c5 = "DATE", c6 = "DATETIME"))
+    on.exit(dbRemoveTable(con, tblName))
+    received <- DBI::dbReadTable(con, tblName)
+    expect_equal(values[-6], received[-6])
+    expect_equal(as.double(values[[6]]), as.double(received[[6]]))
   })
 
   test_that("dates should always be interpreted in the system time zone (#398)", {
     con <- DBItest:::connect(DBItest:::get_default_context(), timezone = "America/Chicago")
-    res <- dbGetQuery(con, "SELECT ?", params = as.Date("2019-01-01"))
-    expect_equal(res[[1]], "2019-01-01")
+    res <- dbGetQuery(con, "SELECT CAST(? AS date)", params = as.Date("2019-01-01"))
+    expect_equal(res[[1]], as.Date("2019-01-01"))
   })
 })
